@@ -1,4 +1,5 @@
 import os
+import sys
 import uuid
 import time
 import multiprocessing
@@ -27,6 +28,7 @@ class Llama:
         n_threads: Optional[int] = None,
         n_batch: int = 8,
         last_n_tokens_size: int = 64,
+        verbose: bool = True,
     ):
         """Load a llama.cpp model from `model_path`.
 
@@ -43,6 +45,7 @@ class Llama:
             n_threads: Number of threads to use. If None, the number of threads is automatically determined.
             n_batch: Maximum number of prompt tokens to batch together when calling llama_eval.
             last_n_tokens_size: Maximum number of tokens to keep in the last_n_tokens deque.
+            verbose: Print verbose output to stderr.
 
         Raises:
             ValueError: If the model path does not exist.
@@ -50,6 +53,7 @@ class Llama:
         Returns:
             A Llama instance.
         """
+        self.verbose = verbose
         self.model_path = model_path
 
         self.params = llama_cpp.llama_context_default_params()
@@ -68,7 +72,7 @@ class Llama:
             maxlen=self.last_n_tokens_size,
         )
         self.tokens_consumed = 0
-        self.n_batch = n_batch
+        self.n_batch = min(n_ctx, n_batch)
 
         self.n_threads = n_threads or multiprocessing.cpu_count()
 
@@ -79,6 +83,9 @@ class Llama:
             self.model_path.encode("utf-8"), self.params
         )
 
+        if self.verbose:
+            print(llama_cpp.llama_print_system_info().decode("utf-8"), file=sys.stderr)
+
     def tokenize(self, text: bytes) -> List[llama_cpp.llama_token]:
         """Tokenize a string.
 
@@ -239,6 +246,10 @@ class Llama:
             An embedding object.
         """
         assert self.ctx is not None
+
+        if self.verbose:
+            llama_cpp.llama_reset_timings(self.ctx)
+
         tokens = self.tokenize(input.encode("utf-8"))
         self.reset()
         self.eval(tokens)
@@ -246,6 +257,10 @@ class Llama:
         embedding = llama_cpp.llama_get_embeddings(self.ctx)[
             : llama_cpp.llama_n_embd(self.ctx)
         ]
+
+        if self.verbose:
+            llama_cpp.llama_print_timings(self.ctx)
+
         return {
             "object": "list",
             "data": [
@@ -296,6 +311,9 @@ class Llama:
         text = b""
         returned_characters = 0
 
+        if self.verbose:
+            llama_cpp.llama_reset_timings(self.ctx)
+
         if len(prompt_tokens) + max_tokens > int(llama_cpp.llama_n_ctx(self.ctx)):
             raise ValueError(
                 f"Requested tokens exceed context window of {llama_cpp.llama_n_ctx(self.ctx)}"
@@ -392,6 +410,9 @@ class Llama:
         if logprobs is not None:
             raise NotImplementedError("logprobs not implemented")
 
+        if self.verbose:
+            llama_cpp.llama_print_timings(self.ctx)
+
         yield {
             "id": completion_id,
             "object": "text_completion",
@@ -480,7 +501,7 @@ class Llama:
         repeat_penalty: float = 1.1,
         top_k: int = 40,
         stream: bool = False,
-    ):
+    ) -> Union[Completion, Iterator[CompletionChunk]]:
         """Generate text from a prompt.
 
         Args:
@@ -586,6 +607,21 @@ class Llama:
         max_tokens: int = 128,
         repeat_penalty: float = 1.1,
     ) -> Union[ChatCompletion, Iterator[ChatCompletionChunk]]:
+        """Generate a chat completion from a list of messages.
+
+        Args:
+            messages: A list of messages to generate a response for.
+            temperature: The temperature to use for sampling.
+            top_p: The top-p value to use for sampling.
+            top_k: The top-k value to use for sampling.
+            stream: Whether to stream the results.
+            stop: A list of strings to stop generation when encountered.
+            max_tokens: The maximum number of tokens to generate.
+            repeat_penalty: The penalty to apply to repeated tokens.
+
+        Returns:
+            Generated chat completion or a stream of chat completion chunks.
+        """
         instructions = """Complete the following chat conversation between the user and the assistant. System messages should be strictly followed as additional instructions."""
         chat_history = "\n".join(
             f'{message["role"]} {message.get("user", "")}: {message["content"]}'
